
#ifndef _SSCMISSUEPLUGIN_H_
#define _SSCMISSUEPLUGIN_H_

// doxygen main page stuff
/*! \mainpage
 
   \section overview_sec Overview
   \section gettingstarted_sec Getting Started
   Start by getting a copy the the SSCMIssuePlugin.h header file, located in your Surround SCM directory under "issueplugins/include/". You will need to implement all 
   of the <a href="globals_func.html">functions</a> defined in this header, in addition you will also need to provide the 6 exported metadata variables outlined in the next section, 
   failure to do so will result in your plugin not being loadable by the Surround SCM server.

   \section pluginmetadata_sec Required plugin metadata

   The following metadata is required to be defined in a source file. Please do not use the UUID in this example, you must to generate your own.
   If you are not sure how, google "uuid generator", there are plenty of free web generators out there.

   \code
   extern "C" 
   { 
      IPExport char SSCMIP_PluginProvider[]  = "FooBar"; 
      IPExport char SSCMIP_PluginUUID[]      = "757ce8d0-56b9-11df-9879-0800500c9a66"; 
      IPExport char SSCMIP_PluginAuthors[]   = "Wesley Workman"; 
      IPExport char SSCMIP_PluginNotes[]     = "This plugin allows Surround SCM to process attachments for FooBar."; 
      IPExport char SSCMIP_PluginVersion[]   = "1.1.5"; 
      IPExport int  SSCMIP_APIVersion        = _SSCMIP_APIVersion_; 
   } 
   \endcode
 
   Note: If above metadata is not provided the plugin will not be loadable by the server.
 
   \section stringencoding_sec String Encoding
   All strings will be provided in UTF-8 format. It is expect that all string provided by the plugin
   will be UTF-8 format (eg connection name, issue id, custom field, etc).
  
   The Surround SCM issue plugin API uses UTF-8 encoding for all string parameters and return values. If 
   you have strings in other unicode encodings, you will need to convert them to UTF-8 before passing them to 
   Surround SCM. Failure to do so may yield unpredictable results.
 
   \section memory_sec Memory
   For each string, list of strings, structure or list of structures that will be returned as an out variable there is a corresponding
   "Free" function that will be called to clean-up that memory sometime after the function has returned. 
   For example the "GetConnectionList()" function will be called, the Surround SCM server will process the list of connections 
   and then the "FreeConnectionList()" function will be called so that list can be cleaned up. It is your responsiblity to implement these
   "Free" functions.
  
   It is recommended that you use purely dynamic memory for out variables.
  
   \section datetime_sec Date/Time
   Date/Time:
   All timestamps will be in GMT.
  
   Date strings should be represented as: YYYY-MM-DD\n
   Date and time strings should be represented as: YYYY-MM-DDTHH:MM:SS
  
   Example: 2010-09-17, 2010-05-19T12:50:01
  
   \section fixing_sec Fixing
   This plugin API offers the ability to "fix" an issue when preforming an attachment. The term "fix" can be 
   thought of loosely as progressing the workflow state of an issue. When the user clicks the "fix" button from 
   a Surround SCM GUI client we will invoke the GetFixWorkflow() function. This will allow the plugin to provide 
   a set of input fields to the user, the idea being information necessary to progress the workflow. Immediately after 
   the user has entered their information we will invoked ValidateWorkflow(), at this point the plugin should varify
   that the user has enter all of the necessary information to progress the workflow, however the workflow should not actually
   be modified until AttachToIssues() is called.
  
   &copy; 2010 Seapine Software, Inc. All rights reserved.
 */

#if defined( WIN32 )
   #define IPExport    __declspec(dllexport)
#elif defined( MACOSX )
   #define IPExport    __attribute__((visibility("default"))) extern
#else
   #define IPExport    extern
#endif

#define _SSCMIP_APIVersion_ 1

#ifdef __cplusplus
extern "C" {
#endif

typedef int SSCMIPResult;
typedef int BOOL;
typedef unsigned int uint;

/*! \enum SSCMIPCustomFieldType
 *
 * This enum is used to identify which UI type a particular custom custom field value is.
 * See SSCMIPCustomField for more information.
 */
enum SSCMIPCustomFieldType
{
   eDropdown = 0, /*!< A non editable drop-down list, aka a combobox. */
   eCheckbox,     /*!< A group of checkboxs. */
   eTextfield,    /*!< A single line text field, aka a lineedit. */
   eTextbody,     /*!< A multi line text field, aka a textbox. */
   eDate,         /*!< A date UI widget. */
   eDateTime,     /*!< A datetime UI widget. */
   eHTMLField     /*!< An HTML formatted widget. */
};


/*! \var SSCMIPResults
 * Status codes that are returned by each function to indicate the success of the operation.
 */
const SSCMIPResult SSCMIP_OK = 0;
const SSCMIPResult SSCMIP_ERROR = 1;


/*! \struct SSCMIPConnectionDetails
 *
 * Provides the configuration of a plugin connection. It is up to you (the plugin developer) 
 * to map a connection name to the necessary details for your plugin to make a connection (eg Hostname, port, project/database, etc).
 * One approach would be for your plugin to read a configuration file that contains connection information.
 *
 * NOTE: Connection names are unique identifiers and are case should be case insensitive.
 *
 */
typedef struct SSCMIPConnectionDetails
{
   /*! The display name for your connection. */
   char *pConnectionName;
   /*! True if this connection can support bulk fix events, false if it cannot.
    *  See GetFixWorkflow for more information.
    */
   BOOL bConnectionSupportsBulkFixEvents;
} SSCMIPConnectionDetails;


/*! \struct SSCMIPConnectionContext
 * 
 * An instance of SSCMIPConnectionContext will be provided as an argument to each interface function call.
 * It provides basic details about the request, to be specific it provides the Username and Password of the
 * user. It also provides the name of the plugin connection.
 */
typedef struct SSCMIPConnectionContext
{
   /*! The username for the connecting user. */
   char *pUsername;
   /*! The password for the connecting user. */
   char *pPassword;
   /*! The connection name for the particular call. This should be ignored for "GetConnectionList()" and "AttachToIssues()". */
   char *pConnection;
} SSCMIPConnectionContext;


/*! \struct SSCMIPFile
 *
 * Provides details about the specific SCM file for the attach operation.
 */
typedef struct SSCMIPFile
{
   /*! Mainline name */
   char *pMainline;
   /*! Branch name */
   char *pBranch;
   /*! Repository path */
   char *pRepoPath;
   /*! Filename */
   char *pFileName;
   /*! Version number for the particular attachment. */
   uint version;
} SSCMIPFile;


/*! \struct SSCMIPIssue
 *
 * This structure is used to provide the basic information about an issue. 
 *
 * The "Type", "Status" and "Description" fields are optional. If your issue tracking software
 * does not provide them you can simply set them to NULL.
 */
typedef struct SSCMIPIssue
{
   /*! The issue id. This should be a unique identifer on a per connection basis. */
   char *pIssueId;
   /*! Brief description or title for the issue. */
   char *pSummary;
   /*! Type field. eg UI Problem, Crash, Data Corruption, etc. */
   char *pType; 
   /*! Status or state the issue is in. eg Fixed, New, Open, Reviewed, etc. */
   char *pStatus;
   /*! A more detailed description of the issue. */
   char *pDescription;
   /*! True if the user can attach to this issue. False otherwise. */
   BOOL bUserCanAttach;
   /*! True if the user can fix this issue. False otherwise. */
   BOOL bUserCanFix;
} SSCMIPIssue;


/*! \struct SSCMIPCustomFieldValue
 *
 *  This structure is used to represent a value for a SSCMIPCustomField structure.
 */
typedef struct SSCMIPCustomFieldValue
{
   /*! The display value is the value that is displayed to the user in the Surround SCM GUI clients. */
   char *pDisplayValue;
   /*! pValueId allows you to provide an internal id that can be used later for reference purposes. (Optional)
       Note: It should not be an index value for a list because it is possible this will be sitting in a database for a long period of time. */
   char *pValueId;
   /*! bSelected is only used for multi-value custom fields, eg eDropdown, eCheckbox. 
       It will be ignored for single-value custom fields, eg eTextfield, eDate, etc.*/
   BOOL bSelected;
} SSCMIPCustomFieldValue;


/*! \struct SSCMIPCustomField
 *
 * This structure is used to provide a custom field. A custom field is used in two different ways. The first is to represent the details of an
 * issue beyond the basic fields provided in SSCMIPIssue, see GetIssueDetails(). The second is to represent a custom workflow field that will
 * be visible to the user when they are fixing an issue, see GetFixWorkflow().
 *
 */
typedef struct SSCMIPCustomField
{
   /*! The display name is the name of the custom field that will be displayed to the user in the Surround SCM GUI clients. */
   char *pDisplayName;
   /*! pFieldId allows you to provide an internal id that can be used later for reference purposes. (Optional)
       Note: It should not be an index value for a list because it is possible this will be sitting in a database for a long period of time. */
   char *pFieldId;
   /*! eType is an enum that indicates the UI widget that should be used for this custom field, see SSCMIPCustomFieldType. */
   enum SSCMIPCustomFieldType eType;
   /*! pCustomFieldValues is a c-style array of custom field values for this custom field. */
   SSCMIPCustomFieldValue **pCustomFieldValues;
   /*! Indicates the number of custom field values that exist in the pCustomFieldValues array. */
   uint numOfFieldValues;
} SSCMIPCustomField;


/*! \struct SSCMIPIssueAttachment
 *
 *  This structure contains the details of an attachment to process, see AttachToIssues().
 */
typedef struct SSCMIPIssueAttachment
{
   /*! The issue id for the current attachment. */
   char *pIssueId;
   /*! The connection name for the current attachment. */
   char *pConnection;
   /*! pFiles is a c-style array of files that are being attached to this issue. */
   SSCMIPFile **pFiles;
   /*! Indicates the number of files that exist in the pFiles array. */
   uint numOfFiles;
   /*! bIsFixed is true if a the current attachment is progressing the workflow for the current issue. */
   BOOL bIsFixed;
   /*! pCustomWorkflows is a c-style array of custom fields that contain the necessary user input to progress the workflow. */
   SSCMIPCustomField **pCustomWorkflows;
   /*! Indicates the number of custom fields that exist in the pCustomWorkflows array. */
   uint numOfWorkflows;
} SSCMIPIssueAttachment;


/*! \struct SSCMIPFailedIssueAttachment
 *
 *  This structure is used to indicate that an issue attachment failed, see AttachToIssues().
 */
typedef struct SSCMIPFailedIssueAttachment
{
   /*! The issue id for the failed attachment. */
   char *pIssueId;
   /*! The connection name for the failed attachment. */
   char *pConnection;
   /*! An error string that is displayed to the user. */
   char *pErrorMsg;
} SSCMIPFailedIssueAttachment;


/*! \brief Called to initialize the plugin.
    
    The Initialize function is called when the plugin is first loaded into memory. This would be an opportunity to open
    any configuration files, read registry settings, load libraries, perform basic checking, etc.
    
    \param[in] pDirectoryPath string that contains the full path to where the plugin is stored on the disk. This is be useful for opening config files or anything that's expected to be in the same directory.
    \param[out] pErrorString string that contains an error message. This should be NULL if the Initialize returns SSCMIP_OK.
    
    \return SSCMIPResult (SSCMIP_OK for success, SSCMIP_ERROR for error)
*/
IPExport SSCMIPResult Initialize(const char *pDirectoryPath, char **pErrorString);


/*! \brief Called when unloading the plugin.
    
    The Uninitialize function is called just before the plugin is unloaded from memory. This would be the time to free any locks and
    clean up memory.
    
*/
IPExport void Uninitialize();





/*! \brief Provides a list of connections.
    
    This function is invoked to get a list of connections. Connections can be different projects, different servera or even  different issue
    trackers. 
 
    The SSCMIPConnectionContext structure contains a boolean called "bConnectionSupportsBulkFixEvents". This should be set to true if that 
    particular connection has the ability to provide provide fix workflow for multiple issues at once. IE if all issues will have the same
    fix workflow or the fix workflow of multiple issue can be combine this should be true. If the connection can only handle generating one fix workflow
    at a time, it should be false.
 
    \param[in] context a pointer to an SSCMIPConnectionContext that contains the user's credentials.
    \param[out] pConnections c-style array containing details for connections provided by the plugin.
    \param[out] numOfConnections indicates the number of connections that exist in the pConnections array.
    \param[out] pErrorString string that contains an error message. This should be NULL if the GetConnectionList returns SSCMIP_OK.
    
    \return SSCMIPResult (SSCMIP_OK for success, SSCMIP_ERROR for error)
 
    Note: The pConnection member on the SSCMIPConnectionContext will be NULL for this function call.
*/
IPExport SSCMIPResult GetConnectionList(const SSCMIPConnectionContext *context,
                                        SSCMIPConnectionDetails **pConnections[],
                                        uint *numOfConnections,
                                        char **pErrorString);





/*! \brief Provides a list of filters.
    
    This function is invoked to get a list of filters for a particular connection. A filter is used to group a set of issues in
    a particular state. This function is optional, if you choose not to implement it or your issue tracking software does not support it
    (*pFilters) should be set to NULL and (*numOfFilters) should be set to 0.
 
    \param[in] context a pointer to an SSCMIPConnectionContext that contains the user's credentials and the connection name.
    \param[out] pFilters c-style string array containing the names of the filters.
    \param[out] numOfFilters indicates the number of filters that exist in the pFilters array.
    \param[out] pErrorString string that contains an error message. This should be NULL if the GetFilterList returns SSCMIP_OK.
    
    \return SSCMIPResult (SSCMIP_OK for success, SSCMIP_ERROR for error)
   */
IPExport SSCMIPResult GetFilterList(const SSCMIPConnectionContext *context,
                                    char **pFilters[],
                                    uint *numOfFilters,
                                    char **pErrorString);





/*! \brief Provides a list of issues for a particular connection and filter.
    
    This function is invoked to get a list of issues for a particular connection and filter. 
   
    \param[in] context a pointer to an SSCMIPConnectionContext that contains the user's credentials and the connection name.
    \param[in] pFilter the name of the filter.
    \param[out] issues c-style array containing the issues.
    \param[out] numOfIssues indicates the number of issues that exist in the "issues" array.
    \param[out] numericIssueIds a bool that should be true if all id's are numeric only. This allows for numeric sorting on the client side.
    \param[out] pErrorString string that contains an error message. This should be NULL if the GetIssueList returns SSCMIP_OK.
    
    \return SSCMIPResult (SSCMIP_OK for success, SSCMIP_ERROR for error)
*/
IPExport SSCMIPResult GetIssueList(const SSCMIPConnectionContext *context,
                                   const char *pFilter,
                                   SSCMIPIssue **issues[],
                                   uint *numOfIssues,
                                   BOOL *numericIssueIds,
                                   char **pErrorString);


   

/*! \brief Provides the details for a specific issue.
    
    This function is invoked to get the details of a specific issue. 
 
    The details for an issue are provided in two ways. The first is via SSCMIPIssue, this contains the basic information about
    an issue. The second way is an array of SSCMIPCustomFields. A SSCMIPCustomField is a customized field that is displayed to the user
    in the UI format you specify, eg dropdown, checkbox, date/time field, etc. See SSCMIPCustomField and SSCMIPCustomFieldType .
    
    \param[in] context a pointer to an SSCMIPConnectionContext that contains the user's credentials and the connection name.
    \param[in] pIssueId the issue id we are requesting the details for.
    \param[out] pIssue the basic details of the issue.
    \param[out] customFields c-style array contains the custom fields that represent the details of a particular issue.
    \param[out] numOfCustomFields indicates the number of custom fields that exist in the "customFields" array.
    \param[out] pErrorString string that contains an error message. This should be NULL if the GetIssueDetails returns SSCMIP_OK.
    
    \return SSCMIPResult (SSCMIP_OK for success, SSCMIP_ERROR for error)
    \sa SSCMIPConnectionContext, SSCMIPIssue, SSCMIPCustomField
*/
IPExport SSCMIPResult GetIssueDetails(const SSCMIPConnectionContext *context,
                                      const char *pIssueId,
                                      SSCMIPIssue **pIssue,
                                      SSCMIPCustomField **customFields[],
                                      uint *numOfCustomFields,
                                      char **pErrorString);





/*! \brief Provides the custom fields necessary to fix an issue or set of issues.
    
    This function is invoked to get the input fields for the user to progress the workflow state of an issue or set of issues.
 
    When providing a list of SSCMIPConnections if a connection has the member "bConnectionSupportsBulkFixEvents" set to true the user
    can highlight multiple issues on a single connection and click fix, in which case numOfIssues will be >= 1. If "bConnectionSupportsBulkFixEvents"
    is set to false, the user can only fix one issue at a time and numOfIssues will be 1 everytime.
 
    \param[in] context a pointer to an SSCMIPConnectionContext that contains the user's credentials and the connection name.
    \param[in] pIssueIds c-style string array that contains the issue ids the user is trying to fix.
    \param[in] numOfIssues indicates the number of issues that exist in the pIssueIds array.
    \param[out] customFields c-style array contains the custom fields that represent necessary input fields to fix an issue or set of issue.
    \param[out] numOfCustomFields indicates the number of custom fields that exist in the "customFields" array.
    \param[out] pErrorString string that contains an error message. This should be NULL if the GetFixWorkflow returns SSCMIP_OK.
    
    \return SSCMIPResult (SSCMIP_OK for success, SSCMIP_ERROR for error)
    \sa SSCMIPConnectionContext, SSCMIPCustomField
 */
IPExport SSCMIPResult GetFixWorkflow(const SSCMIPConnectionContext *context,
                                     const char *pIssueIds[],
                                     uint numOfIssues,
                                     SSCMIPCustomField **customFields[],
                                     uint *numOfCustomFields,
                                     char **pErrorString); 





/*! \brief Called to process attachments.
    
    This function is invoked to process Surround SCM file attachments for a set of issues.

    If an attachment fails for any reason an instance of SSCMIPFailedIssueAttachment should be provided in the out variable "failedAttachments".
    
    \param[in] context a pointer to an SSCMIPConnectionContext that contains the user's credentials and the connection name.
    \param[in] issueAttachments c-style array that contains the attachments to process.
    \param[in] numOfAttachments indicates the number of attachments that exist in the issueAttachments array.
    \param[out] failedAttachments c-style array that contains the attachments that failed.
    \param[out] numOfFailedAttachments indicates the number of failed attachments that exist in the "failedAttachments" array.
    \param[out] pErrorString string that contains an error message. This should be NULL if the AttachToIssues returns SSCMIP_OK.
    
    \return SSCMIPResult (SSCMIP_OK for success, SSCMIP_ERROR for error other than failed attachment)
    \sa SSCMIPConnectionContext, SSCMIPCustomField
 
    Note: The pConnection member on the SSCMIPConnectionContext will be NULL for this function call, you should instead check the pConnection member 
    on the SSCMIPIssueAttachment. It is possible for the user to make attachments from multiple connections.
 
*/
IPExport SSCMIPResult AttachToIssues(const SSCMIPConnectionContext *context,
                                     const SSCMIPIssueAttachment *issueAttachments[],
                                     uint numOfAttachments,
                                     SSCMIPFailedIssueAttachment **failedAttachments[],
                                     uint *numOfFailedAttachments,
                                     char **pErrorString);





/*! \brief Validates the workflow entered by the user.
    
    This function is invoked to validate the user's input values when fixing an issue or set of issues. This function should check all custom field values to verify
    that satisfy the requirements necessary to fix an issue or set of issues.
    
    When providing a list of SSCMIPConnections if a connection has the member "bConnectionSupportsBulkFixEvents" set to true the user
    can highlight multiple issues on a single connection and click fix, in which case numOfIssues will be >= 1. If "bConnectionSupportsBulkFixEvents"
    is set to false, the user can only fix one issue at a time and numOfIssues will be 1 everytime.
 
    If the workflow is valid (*workflowIsValid) should be set to true. If it is invalid (*workflowIsValid) should be set to false and *pErrorString should
    be populated with an explanation as to why it is not valid, this will be provided to the user.
 
    \param[in] context a pointer to an SSCMIPConnectionContext that contains the user's credentials and the connection name.
    \param[in] pIssueIds c-style string array that contains the issue ids the user is trying to fix.
    \param[in] numOfIssues indicates the number of issues that exist in the pIssueIds array.
    \param[in] customFields c-style array contains the custom fields that represent the user's input for the workflow provided by GetFixWorkflow.
    \param[in] numOfCustomFields indicates the number of custom fields that exist in the "customFields" array.
    \param[out] workflowIsValid true if the workflow is valid, false if it is invalid.
    \param[out] pErrorString string that contains an error message. This should be NULL if the ValidateWorkflow returns SSCMIP_OK and if (*workflowIsValid) is true.
    
    \return SSCMIPResult (SSCMIP_OK for success, SSCMIP_ERROR for error other than invalid workflow)
    \sa SSCMIPConnectionContext, SSCMIPCustomField
    
    Note: The pConnection member on the SSCMIPConnectionContext will be NULL for this function call, you should instead check the pConnection member 
    on the SSCMIPIssueAttachment. It is possible for the user to make attachments from multiple connections.
 
    Note: You should only validate the workflow during this call, do not actually up the workflow. Updating the workflow should take place in AttachToIssues. 
    It is possible for the user to put their workflow on a changelist for any length of time.
    
*/
IPExport SSCMIPResult ValidateWorkflow(const SSCMIPConnectionContext *context,
                                       const char *pIssueIds[],
                                       uint numOfIssues,
                                       const SSCMIPCustomField *customFields[], 
                                       uint numOfCustomFields,
                                       BOOL *workflowIsValid,
                                       char **pErrorString);






/*! \brief This function is called to free a connection list.
 
    Because your plugin returns dynamically allocated memory it needs to be the one to free it.
 
    \sa SSCMIPConnectionDetails
 */
IPExport void FreeConnectionList(SSCMIPConnectionDetails *connections[], uint numOfConnections);




/*! \brief This function is called to free a custom field list.
    
    Because your plugin returns dynamically allocated memory it needs to be the one to free it.
 
    \sa SSCMIPCustomField
*/
IPExport void FreeCustomFieldsList(SSCMIPCustomField *customFields[], uint numOfCustomFields);




/*! \brief This function is called to free a failed attachment list.
    
    Because your plugin returns dynamically allocated memory it needs to be the one to free it.
 
    \sa SSCMIPFailedIssueAttachment
*/
IPExport void FreeFailedAttachmentList(SSCMIPFailedIssueAttachment *failedAttachments[], uint numOfFailedAttachments);




/*! \brief This function is called to free an issue.
    
   Because your plugin returns dynamically allocated memory it needs to be the one to free it.
    
   \sa SSCMIPIssue
*/
IPExport void FreeIssue(SSCMIPIssue *issue);




/*! \brief This function is called to free an issue list.
    
    Because your plugin returns dynamically allocated memory it needs to be the one to free it.
 
    \sa SSCMIPIssue
*/
IPExport void FreeIssueList(SSCMIPIssue *issues[], uint numOfIssues);




/*! \brief This function is called to free a string list.
    
    Because your plugin returns dynamically allocated memory it needs to be the one to free it.
*/
IPExport void FreeStringList(char *pStrList[], uint numOfStrings);




/*! \brief This function is called to free a string.
    
    Because your plugin returns dynamically allocated memory it needs to be the one to free it.
*/
IPExport void FreeString(char *pStr);




#ifdef __cplusplus
}
#endif

#endif /* _SSCMISSUEPLUGIN_H_ */

